begin
  require_dependency 'factory_girl'
  FactoryGirl.find_definitions
rescue LoadError
rescue FactoryGirl::DuplicateDefinitionError
end

begin
  require_dependency 'factory_bot'
  FactoryBot.find_definitions
rescue LoadError
rescue FactoryBot::DuplicateDefinitionError
end

module CrossSpecRails
  class FactoriesController < ApplicationController
    skip_before_action :verify_authenticity_token

    def create
      instance = factory_klass.create(factory, *traits, attributes)
      render json: instance.to_json(to_json_args)
    end

    def create_list
      instances = factory_klass.create_list(factory, num, *traits, attributes)
      render json: instances.map { |instance| instance.to_json(to_json_args) }
    end

    private

    def factory
      params[:factory]
    end

    def traits
      Array(params[:traits]).map(&:to_sym)
    end

    def attributes
      params[:attributes]&.permit!.to_h
    end

    def to_json_args
      params[:to_json_args]
    end

    def num
      params[:num].to_i
    end

    def factory_klass
      return FactoryBot if defined? FactoryBot
      return FactoryGirl if defined? FactoryGirl
      raise "Unable to find a factory builder class"
    end
  end
end
